@@ -1,20 +1,21 @@
 .onAttach <- function(lib,pkg) {
   local_version <- utils::packageVersion("backbone")
   packageStartupMessage(" ____   backbone v",local_version)
-  packageStartupMessage("|  _ \\  Cite: Domagalski, R., Neal, Z. P., & Sagan, B. (2021). Backbone: An")
-  packageStartupMessage("|#|_) |       R package for extracting the backbone of bipartite projections. ")
-  packageStartupMessage("|# _ <        PLoS ONE, 16(1), e0244363. https://doi.org/10.1371/journal.pone.0244363")
+  packageStartupMessage("|  _ \\  Cite: Neal, Z. P., (2022). Backbone: An R package to extract network backbones.")
+  packageStartupMessage("|#|_) |       PLOS ONE, 17, e0269137. https://doi.org/10.1371/journal.pone.0269137")
+  packageStartupMessage("|# _ < ")
   packageStartupMessage("|#|_) | Help: type vignette(\"backbone\"); email zpneal@msu.edu; github zpneal/backbone")
   packageStartupMessage("|____/  Beta: type devtools::install_github(\"zpneal/backbone\", ref = \"devel\")")
 }
 
 #' Converts an input graph object to an adjacency/incidence matrix and identifies its characteristics
 #'
-#' @param graph A graph object of class "matrix", "sparseMatrix", "dataframe", \link[igraph]{igraph}, \link[network]{network}.
+#' @param graph A graph object of class "matrix", "sparseMatrix", "dataframe", \link[igraph]{igraph}.
 #'
-#' @return a list(summary, G)
+#' @return a list(summary, G, attribs)
 #'    `summary` is a dataframe containing characteristics of the supplied object
 #'    `G` is an adjacency/incidence matrix
+#'    `attribs` is a dataframe containing node characteristics if present in `graph`
 #'
 #' @keywords internal
 #'
@@ -22,19 +23,19 @@
 #' M <- matrix(rbinom(5*5,1,.5),5,5)
 #' test <- backbone:::tomatrix(M)
 tomatrix <- function(graph){
-  class <- class(graph)[1]
+  class <- class(graph)[1]  #Identify class of supplied object
   isbipartite <- FALSE
 
-  if (!(methods::is(graph, "matrix")) & !(methods::is(graph, "sparseMatrix")) & !(methods::is(graph, "Matrix")) & !(methods::is(graph, "igraph")) & !(methods::is(graph, "network")) & !(methods::is(graph, "data.frame"))) {stop("input bipartite data must be a matrix, edgelist dataframe, igraph, or network object.")}
+  if (!(methods::is(graph, "matrix")) & !(methods::is(graph, "Matrix")) & !(methods::is(graph, "igraph")) & !(methods::is(graph, "data.frame"))) {stop("input bipartite data must be a matrix, Matrix, dataframe, or igraph object.")}
 
-  #### Convert from matrix object ####
-  if (((methods::is(graph, "matrix")) | (methods::is(graph, "sparseMatrix")) | (methods::is(graph, "Matrix")))) {
+  #### Convert from matrix or Matrix object ####
+  if (((methods::is(graph, "matrix")) | (methods::is(graph, "Matrix")))) {
     G <- as.matrix(graph)  #Coerce to matrix
     class(G) <- "numeric"  #Coerce to numeric
     if (any(is.na(G))) {stop("The object contains non-numeric entries")}
 
     if (dim(G)[1]!=dim(G)[2]) {isbipartite <- TRUE}  #A rectangular matrix is treated as bipartite
-    if (dim(G)[1]==dim(G)[2] & !is.null(rownames(G)) & !is.null(colnames(G))) { #A labeled square matrix is treated as bipartite IFF
+    if (dim(G)[1]==dim(G)[2] & !is.null(rownames(G)) & !is.null(colnames(G))) { #A labeled square matrix is treated as bipartite if
       if (!identical(rownames(G),colnames(G)) &                                 #the row and column labels differ, and
           !isSymmetric(G)) {                                                    #it is not symmetric
         isbipartite <- TRUE
@@ -44,7 +45,7 @@ tomatrix <- function(graph){
 
   #### Convert from edge list ####
   if (methods::is(graph, "data.frame")) {
-    if (ncol(graph)==1 | ncol(graph)>3) {stop("An edgelist must contain 2 or 3 columns")}
+    if (ncol(graph)<2 | ncol(graph)>3) {stop("An edgelist must contain 2 or 3 columns")}
     class <- "edgelist"  #Update starting class as edgelist
     G <- graph
     colnames(G) <- LETTERS[1:dim(graph)[2]]  #Name columns A, B, and (if present) C
@@ -67,43 +68,54 @@ tomatrix <- function(graph){
   #### Convert from igraph ####
   if (methods::is(graph, "igraph")) {
 
+    graph <- igraph::simplify(graph) #Remove any multi-edges and loops
+
     if (igraph::is.bipartite(graph) == TRUE){ #Bipartite
-      isbipartite <- TRUE
-      if (is.null(igraph::E(graph)$weight)) {G <- igraph::as_incidence_matrix(graph, sparse = FALSE) #Unweighted
-      } else {G <- igraph::as_incidence_matrix(graph, attr = "weight", sparse = FALSE)} #Weighted
+      isbipartite <- TRUE  #Set type of graph
+
+      #Remove isolates and maximally connected nodes; similar as code below to remove empty/full rows/columns
+      isolates <- which(igraph::degree(graph)==0)  #Isolate nodes
+      fullrows <- which(igraph::degree(graph, v = igraph::V(graph)$type==FALSE) == sum(igraph::V(graph)$type==TRUE))  #FALSE nodes connected to all TRUE nodes
+      fullcols <- which(igraph::degree(graph, v = igraph::V(graph)$type==TRUE) == sum(igraph::V(graph)$type==FALSE))  #TRUE nodes connected to all FALSE nodes
+      todrop <- c(isolates, fullrows, fullcols)
+      while (length(todrop)>0) {
+        graph <- igraph::delete.vertices(graph, todrop)
+        isolates <- which(igraph::degree(graph)==0)  #Isolate nodes
+        fullrows <- which(igraph::degree(graph, v = igraph::V(graph)$type==FALSE) == sum(igraph::V(graph)$type==TRUE))  #FALSE nodes connected to all TRUE nodes
+        fullcols <- which(igraph::degree(graph, v = igraph::V(graph)$type==TRUE) == sum(igraph::V(graph)$type==FALSE))  #TRUE nodes connected to all FALSE nodes
+        todrop <- c(isolates, fullrows, fullcols)
+      }
+
+      #Capture attributes of FALSE (i.e. row) nodes
+      attribs <- as.data.frame(igraph::vertex_attr(graph, index = (igraph::V(graph)$type == FALSE)))  #Get attributes
+      attribs <- attribs[,!colnames(attribs)%in%c("type","name"),drop=F]  #Do not need to keep type or name
+      attribs <- Filter(function(x)!all(is.na(x)), attribs)  #Remove unused attributes
+      if (ncol(attribs)==0) {rm(attribs)}  #Delete dataframe if empty
+
+      #Convert to incidence matrix
+      if (is.null(igraph::E(graph)$weight)) {G <- igraph::as_incidence_matrix(graph, sparse = FALSE) #Unweighted bipartite
+        } else {G <- igraph::as_incidence_matrix(graph, attr = "weight", sparse = FALSE)} #Weighted bipartite
     }
 
     if (igraph::is.bipartite(graph) == FALSE){ #Unipartite
-      if (is.null(igraph::E(graph)$weight)) {G <- igraph::as_adjacency_matrix(graph, sparse = FALSE) #Unweighted
-      } else {G <- igraph::as_adjacency_matrix(graph, attr = "weight", sparse = FALSE)} #Weighted
+
+      #Capture attributes of nodes
+      attribs <- as.data.frame(igraph::vertex_attr(graph))  #Get attributes
+      attribs <- attribs[,!colnames(attribs)%in%c("type","name"),drop=F]  #Do not need to keep type or name
+      attribs <- Filter(function(x)!all(is.na(x)), attribs)  #Remove unused attributes
+      if (ncol(attribs)==0) {rm(attribs)}  #Delete dataframe if empty
+
+      #Convert to adjacency matrix
+      if (is.null(igraph::E(graph)$weight)) {G <- igraph::as_adjacency_matrix(graph, sparse = FALSE) #Unweighted unipartite
+      } else {G <- igraph::as_adjacency_matrix(graph, attr = "weight", sparse = FALSE)} #Weighted unipartite
     }
   }
 
-  #### Convert from statnet ####
-  if (methods::is(graph, "network")) {
-
-    if (network::is.bipartite(graph) == TRUE) { #Bipartite
-      isbipartite <- TRUE
-      if ("weight" %in% network::list.edge.attributes(graph)) {
-        G <- network::as.matrix.network(graph, type = "incidence", attrname = "weight")} else { #Weighted
-          G <- network::as.matrix.network(graph, type = "incidence")} #Unweighted
-    }
-
-    if (network::is.bipartite(graph) == FALSE) { #Unipartite
-      if ("weight" %in% network::list.edge.attributes(graph)) {
-        G <- network::as.matrix.network(graph, type = "adjacency", attrname = "weight")} else { #Weighted
-          G <- network::as.matrix.network(graph, type = "adjacency")} #Unweighted
-    }
-  }
-
-  #### If the graph is bipartite, remove rows/columns with zero sums ####
+  #### If the graph is bipartite, remove rows/columns with zero sums (already done above if igraph ####
   if (isbipartite){
-    R <- Matrix::rowSums(G)
-    C <- Matrix::colSums(G)
-    r <- which(R == 0)
-    c <- which(C == 0)
-    if (length(r)>0){G <- G[-r,]}
-    if (length(c)>0){G <- G[,-c]}
+    while (any(rowSums(G)==0) | any(rowSums(G)==ncol(G)) | any(colSums(G)==0) | any(colSums(G)==nrow(G))) {  #If any rows/columns are empty/full
+      G <- G[which(rowSums(G)!=0 & rowSums(G)!=ncol(G)),which(colSums(G)!=0 & colSums(G)!=nrow(G))]  #Remove them, check again
+    }
   }
 
   #### Summary dataframe ####
@@ -115,23 +127,14 @@ tomatrix <- function(graph){
     symmetric = issymmetric,
     weighted = isweighted)
 
-  #### Report input type and modifications ####
-  if (issymmetric) {dir <- "undirected"} else {dir <- "directed"}
-  if (isweighted) {weigh <- "a weighted"} else {weigh <- "an unweighted"}
-  if (isbipartite) {
-    message(paste0("This ", class, " object is being treated as ", weigh, " bipartite network of ", nrow(G), " agents and ", ncol(G), " artifacts."))
-    if (length(r)>0) {message("These empty (i.e. all 0s) rows have been removed from the data: ", paste0(r, " "))}
-    if (length(c)>0) {message("These empty (i.e. all 0s) columns have been removed from the data: ", paste0(c, " "))}
-  }
-  if (!isbipartite) {message(paste0("This ", class, " object is being treated as ", weigh, " ", dir, " network containing ", nrow(G), " nodes."))}
-
-  return(list(summary = summary, G = G))
+  if (exists("attribs")) {return(list(summary = summary, G = G, attribs = attribs))} else {return(list(summary = summary, G = G))}
 }
 
 #' Converts a backbone adjacency matrix to an object of specified class
 #'
 #' @param graph a matrix
-#' @param convert class to convert to, one of "matrix", "sparseMatrix", "igraph", "edgelist", or "network"
+#' @param attribs dataframe: vertex attributes to be assigned in igraph object
+#' @param convert class to convert to, one of "matrix", "Matrix", "igraph", or "edgelist"
 #'
 #' @return backbone graph: Binary or signed backbone graph of class given in parameter `convert`.
 #'
@@ -140,19 +143,12 @@ tomatrix <- function(graph){
 #' @examples
 #' M <- matrix(sample(c(-1,0,1),5*5,replace=TRUE),5,5)
 #' test <- backbone:::frommatrix(M, "Matrix")
-frommatrix <- function(graph, convert = "matrix"){
+frommatrix <- function(graph, attribs = NA, convert = "matrix"){
 
   if (convert == "matrix"){G <- graph}
 
   if (convert == "Matrix"){G <- Matrix::Matrix(graph)}
 
-  if (convert == "sparseMatrix"){G <- methods::as(graph, "sparseMatrix")}
-
-  if (convert == "network"){
-    if (isSymmetric(graph)) {G <- network::network(graph, ignore.eval = FALSE, names.eval = "weight", directed = FALSE)}
-    if (!isSymmetric(graph)) {G <- network::network(graph, ignore.eval = FALSE, names.eval = "weight", directed = TRUE)}
-  }
-
   if (convert == "edgelist"){
     if (isSymmetric(graph)) {G <- igraph::graph.adjacency(graph, mode = "undirected", weighted = TRUE)}
     if (!isSymmetric(graph)) {G <- igraph::graph.adjacency(graph, mode = "directed", weighted = TRUE)}
@@ -163,6 +159,14 @@ frommatrix <- function(graph, convert = "matrix"){
     if (isSymmetric(graph)) {G <- igraph::graph.adjacency(graph, mode = "undirected", weighted = TRUE)}
     if (!isSymmetric(graph)) {G <- igraph::graph.adjacency(graph, mode = "directed", weighted = TRUE)}
     if (igraph::gsize(G)!=0) {igraph::E(G)$sign <- igraph::E(G)$weight}  #To facilitate use with library(signnet)
+
+    if (!is.null(attribs)) {  #If attributes are supplied, add them to the igraph object
+      attribs.to.add <- colnames(attribs)
+      for (attrib in attribs.to.add) {
+        G <- igraph::set_vertex_attr(graph = G, name = attrib, value = attribs[,attrib])
+      }
+    }
+
   }
 
   return(G)
@@ -177,7 +181,7 @@ frommatrix <- function(graph, convert = "matrix"){
 #' @param signed Boolean: TRUE for a signed backbone, FALSE for a binary backbone (see details)
 #' @param alpha Real: significance level of hypothesis test(s)
 #' @param mtc string: type of Multiple Test Correction to be applied; can be any method allowed by \code{\link{p.adjust}}.
-#' @param class string: the class of the returned backbone graph, one of c("matrix", "sparseMatrix", "igraph", "network", "edgelist"), converted via \link{tomatrix}.
+#' @param class string: the class of the returned backbone graph, one of c("matrix", "sparseMatrix", "igraph", "edgelist"), converted via \link{tomatrix}.
 #' @return backbone graph: Binary or signed backbone graph of class given in parameter `class`.
 #'
 #' @details The "backbone" S3 class object is composed of three matrices (the weighted graph, edges' upper-tail p-values,
@@ -214,9 +218,8 @@ backbone.extract <- function(bb.object, signed = FALSE, alpha = 0.05, mtc = "non
       & (class != "Matrix")
       & (class != "sparseMatrix")
       & (class != "igraph")
-      & (class != "network")
       & (class != "edgelist"))
-  {stop("incorrect class type, must be one of c(matrix, Matrix, sparseMatrix, igraph, network, edgelist)")}
+  {stop("incorrect class type, must be one of c(matrix, Matrix, sparseMatrix, igraph, edgelist)")}
 
   #### Extract object components ####
   G <- bb.object$G
@@ -269,7 +272,7 @@ backbone.extract <- function(bb.object, signed = FALSE, alpha = 0.05, mtc = "non
   }
 
   #### Return result ####
-  backbone <- frommatrix(backbone, class)
+  backbone <- frommatrix(backbone, convert = class)
   return(backbone)
 }
 
@@ -302,3 +305,36 @@ fastball <- function(M, trades = 5 * nrow(M)) {
     return(Mrand)
   } else {return(fastball_cpp(M, 5 * length(M)))}
 }
+
+#' Poisson binomial distribution function
+#'
+#' `pb` computes the poisson binomial distribution function using the refined normal approximation.
+#'
+#' @param k numeric: values where the pdf should be evaluated
+#' @param p numeric: vector of success probabilities
+#' @param lower boolean: If FALSE return lower tail, if FALSE return upper tail
+#'
+#' @details
+#' The Refined Normal Approximation (RNA) offers a close approximation when `length(p)` is large (Hong, 2013). This function
+#'    is a slightly more efficient implementation of `ppoibin()` from the `poibin` package.
+#'
+#' @return numeric: probability of observing `k` or fewer (if lower = TRUE), or more than `k` (if lower = FALSE),
+#'    successes when each trial has probability `p` of success
+#'
+#' @references
+#' {Hong, Y. (2013). On computing the distribution function for the Poisson binomial distribution. *Computational Statistics and Data Analysis, 59*, 41-51. \doi{10.1016/j.csda.2012.10.006}}
+#'
+#' @export
+#'
+#' @examples
+#' pb(50,runif(100))
+pb <-function(k,p,lower=TRUE) {
+  mu <- sum(p)
+  sigma <- sqrt(sum(p*(1-p)))
+  gamma <- sum(p*(1-p)*(1-2*p))
+  k <- (k+.5-mu)/sigma
+  prob <- stats::pnorm(k)+gamma/(6*sigma^3)*(1-k^2)*stats::dnorm(k)
+  prob[prob<0] <- 0
+  prob[prob>1] <- 1
+  if (lower) {return(prob)} else {return(1-prob)}
+}
